import addonHandler
addonHandler.initTranslation()

import api
import wx
import ui
import textInfos
import sayAllHandler
import speech
from .ww_msg  import NVDAString
from ww_utils import printDebug
import time
import gui
import queueHandler
from NVDAObjects.window.winword import *
from ww_fields import Field


class BrowseModeTreeInterceptorEx(browseMode.BrowseModeTreeInterceptor):

	scriptCategory = _("Extended browse mode for Microsoft Word")
	@classmethod
	def addQuickNav(cls, itemType, key, nextDoc, nextError, prevDoc, prevError, readUnit=None):
		super(BrowseModeTreeInterceptorEx, cls).addQuickNav(itemType, key, nextDoc, nextError, prevDoc, prevError, readUnit)
		

# Add quick navigation scripts.
qn = BrowseModeTreeInterceptorEx.addQuickNav
qn("annotation", key="a",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=NVDAString("moves to the next annotation"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=NVDAString("no next annotation"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=NVDAString("moves to the previous annotation"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=NVDAString("no previous annotation"))
	
qn("grammaticalError", key="$",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next grammatical error"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next grammatical error"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous grammatical error"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous grammatical error"))


qn("comment", key="j",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next comment"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next comment"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous comment"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous comment"))

qn("field", key="y",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next field"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next field"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous field"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous field"),
	readUnit=textInfos.UNIT_LINE)

qn("bookmark", key=";",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next bookmark"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next bookmark"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous bookmark"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous bookmark"))




qn("endnote", key="!",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next endnote"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next endnote"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous endnote"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous endnote"))
qn("footnote", key=":",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next footnote"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next footnote"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous footnote"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous footnote"))
	
qn("spellingError", key="*",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next spelling error"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next spelling error"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous spelling error"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous spelling error"))

qn("revision", key="<",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next revision"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next revision"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous revision"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous revision"))

qn("section", key=")",
	# Translators: Input help message for a quick navigation command in browse mode.
	nextDoc=_("moves to the next section"),
	# Translators: Message presented when the browse mode element is not found.
	nextError=_("no next section"),
	# Translators: Input help message for a quick navigation command in browse mode.
	prevDoc=_("moves to the previous section"),
	# Translators: Message presented when the browse mode element is not found.
	prevError=_("no previous section"))




del qn
class BrowseModeDocumentTreeInterceptorEx(BrowseModeTreeInterceptorEx, browseMode.BrowseModeDocumentTreeInterceptor):
	pass
	


class WordDocumentTreeInterceptorEx(BrowseModeDocumentTreeInterceptorEx, WordDocumentTreeInterceptor):
	TextInfo=BrowseModeWordDocumentTextInfo
	def __init__(self,obj):
		super(WordDocumentTreeInterceptorEx,self).__init__(obj)
		try:
			self.removeGestureBinding("kb:alt+upArrow")
			self.removeGestureBinding("kb:alt+downArrow")
			self.removeGestureBinding("kb:control+upArrow")
			self.removeGestureBinding("kb:control+downArrow")
		except:
			pass
	
	def _get_ElementsListDialog(self):
		return ElementsListDialog

	def _iterNodesByType(self,nodeType,direction="next",pos=None):
		if pos:
			rangeObj=pos.innerTextInfo._rangeObj 
		else:
			rangeObj=self.rootNVDAObject.WinwordDocumentObject.range(0,0)
		includeCurrent=False if pos else True
		if nodeType=="link":
			return LinkWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="annotation":
			comments=CommentWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			revisions=RevisionWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			return browseMode.mergeQuickNavItemIterators([comments,revisions],direction)

		elif nodeType=="bookmark":
			return BookmarkWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="comment":
			return CommentWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="endnote":
			return EndnoteWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="field":
			fields = FieldWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			formfields=FormFieldWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			return browseMode.mergeQuickNavItemIterators([fields,formfields],direction)
		elif nodeType=="formfield":
			return FormFieldWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()

		elif nodeType=="footnote":
			return FootnoteWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="grammaticalError":
			return GrammaticalErrorWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="revision":
			return RevisionWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="spellingError":
			return SpellingErrorWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="table":
			return TableWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType=="graphic":
			shapes=ShapeWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			inLineShapes=InLineShapeWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
			return browseMode.mergeQuickNavItemIterators([shapes,inLineShapes],direction)

		elif nodeType=="section":
			return SectionWinWordCollectionQuicknavIterator(nodeType,self,direction,rangeObj,includeCurrent).iterate()
		elif nodeType.startswith('heading'):
			return self._iterHeadings(nodeType,direction,rangeObj,includeCurrent)
		else:
			raise NotImplementedError

	
class WinWordCollectionQuicknavIteratorEx (WinWordCollectionQuicknavIterator):
	# rewritten cause of bug 
	def iterate(self):
		"""
		returns a generator that emits L{QuickNavItem} objects for this collection.
		"""
		if self.direction=="next":
			self.rangeObj.moveEnd(wdStory,1)
		elif self.direction=="previous":
			self.rangeObj.collapse(wdCollapseStart)
			self.rangeObj.moveStart(wdStory,-1)
		items=self.collectionFromRange(self.rangeObj)
		# my modification
		itemCount=items.count if items != None  else 0

		isFirst=True
		for index in xrange(1,itemCount+1):
			if self.direction=="previous":
				index=itemCount-(index-1)
			collectionItem=items[index]
			item=self.quickNavItemClass(self.itemType,self.document,collectionItem)
			itemRange=item.rangeObj
			# Skip over the item we're already on.
			if not self.includeCurrent and isFirst and ((self.direction=="next" and itemRange.start<=self.rangeObj.start) or (self.direction=="previous" and itemRange.end>self.rangeObj.end)):
				continue
			if not self.filter(collectionItem):
				continue
			yield item
			isFirst=False




class WordDocumentCommentQuickNavItem(WordDocumentCollectionQuickNavItem):
	

	@property
	def label(self):

		author=self.collectionItem.author
		date=self.collectionItem.date
		text=self.collectionItem.range.text
		if text == None:
			text = ""
		return NVDAString(u"comment: {text} by {author} on {date}").format(author=author,text=text,date=date)

	def rangeFromCollectionItem(self,item):
		return item.scope


class CommentWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentCommentQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.comments

class WordDocumentBookmarkQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		item = self.collectionItem
		text = ""
		if not item.empty:
			text = item.range.text
		name = item.Name  if item.Name else _("Bookmark")
		return _("{name} text: {text}") .format(name = name, text= text)
	def report(self,readUnit=None):
		ui.message(_("Bookmark %s" %self.collectionItem.index))



class BookmarkWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentBookmarkQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.Bookmarks

class WordDocumentEndnoteQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		return _("Endnote{index}").format(index =self.collectionItem.index) 
		
	def rangeFromCollectionItem(self,item):
		return item.Reference
	def report(self,readUnit=None):
		ui.message(_("Endnote {index}") .format(index = self.collectionItem.index))



class EndnoteWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentEndnoteQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.Endnotes
class WordDocumentFieldQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		typeText = Field._getTypeText(self.collectionItem.type)
		return _(u"Field {index}, type: {type}").format(index =self.collectionItem.index, type = typeText) 
	def rangeFromCollectionItem(self,item):
		return item.Code


class FieldWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentFieldQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.fields


	
			
class WordDocumentFormFieldQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
	
		return u"{name}".format(name =self.collectionItem.Name) 

	
class FormFieldWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentFormFieldQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.FormFields



	

class WordDocumentFootnoteQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		return _("Footnote {index}").format(index =self.collectionItem.index) 
	def rangeFromCollectionItem(self,item):
		return item.Reference
			
	def report(self,readUnit=None):
		ui.message(_("Footnote {index}") .format(index = self.collectionItem.index))


	def moveTo(self):
		info=self.textInfo.copy()
		info.collapse()
		self.document._set_selection(info,reason=browseMode.REASON_QUICKNAV)


class FootnoteWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentFootnoteQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.Footnotes



class WordDocumentShapeQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		return u"%s" %self.collectionItem.name
	def rangeFromCollectionItem(self,item):
		return item.Anchor
	def report(self,readUnit=None):
		ui.message(_("%s" %self.collectionItem.Name))



class ShapeWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentShapeQuickNavItem
	def collectionFromRange(self,rangeObj):
		try:
			return rangeObj.ShapeRange
		except:
			return None

	

class WordDocumentInLineShapeQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		return _(u"Text layer  ")
	#def report(self,readUnit=None):
		#ui.message(_("Text layer object " 



class InLineShapeWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentInLineShapeQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.InLineShapes

class WordDocumentSpellingErrorQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		return _(u"{text}").format(text =self.collectionItem.text) 
		
	def rangeFromCollectionItem(self,item):
		return item

		

	


	

class SpellingErrorWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentSpellingErrorQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.SpellingErrors
		
	

class WordDocumentGrammaticalErrorQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):
		text = self.collectionItem.text
		if len(text) > 100:
			text = "%s ..." %text[:100]
		return _(u"{text}").format(text =text) 
		
	def rangeFromCollectionItem(self,item):
		return item

		

class GrammaticalErrorWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentGrammaticalErrorQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.GrammaticalErrors
		
	
			
class WordDocumentSectionQuickNavItem(WordDocumentCollectionQuickNavItem):
	@property
	def label(self):

		return _(u"Section {index}").format(index =self.collectionItem.index) 
	def report(self,readUnit=None):
		ui.message(_("Section %s" %self.collectionItem.index))




class SectionWinWordCollectionQuicknavIterator(WinWordCollectionQuicknavIteratorEx):
	quickNavItemClass=WordDocumentSectionQuickNavItem
	def collectionFromRange(self,rangeObj):
		return rangeObj.Sections


			

class ElementsListDialog(wx.Dialog):

	ELEMENT_TYPES=(
		# Translators: The label of a list item to select the type of element
		# in the browse mode Elements List dialog.
		("heading", _("Heading")),
		("link", _("Link")),
		("bookmark", _("Bookmark")),
		("comment", _("Comment")),
		("endnote", _("Endnote")),
		("field", _("Field")),
		("formfield", _("FormField")),
		("footnote", _("Footnote")),
		("graphic", _("Graphic")),
		("revision", _("Revision")),
		("section", _("Section")),
		("spellingError", _("Spelling error")),
#		("grammaticalError", _("Grammatical error")),
	)
	Element = collections.namedtuple("Element", ("item", "parent"))
	lastSelectedElementType=0
	_timer = None
	
	def __init__(self, document):
		self.document = document
		# Translators: The title of the browse mode Elements List dialog.
		super(ElementsListDialog, self).__init__(gui.mainFrame, wx.ID_ANY, NVDAString("Elements List"))
		mainSizer = wx.GridBagSizer(5,5) 
		# Translators: The label of a list of items to select the type of element
		# in the browse mode Elements List dialog.
		childSizer=wx.BoxSizer(wx.VERTICAL)
		childLabel=wx.StaticText(self,wx.NewId(),label= NVDAString("&Type:") , style =wx.ALIGN_CENTRE )
		childSizer.Add(childLabel, )
		self.childListBox =wx.ListBox(self,wx.ID_ANY,name= "TypeName" ,choices=tuple(et[1] for et in self.ELEMENT_TYPES),  style = wx.LB_SINGLE ,size= (596,130))
		if self.childListBox.GetCount():
			self.childListBox.SetSelection(self.lastSelectedElementType)
		self.childListBox.Bind(wx.EVT_LISTBOX, self.onElementTypeChange)
		childSizer.Add(self.childListBox)
		mainSizer.Add(childSizer,(0,0), (1,5), flag= wx.EXPAND|wx.ALL,border= 8) 

		self.tree = wx.TreeCtrl(self, wx.ID_ANY, style=wx.TR_HAS_BUTTONS | wx.TR_HIDE_ROOT | wx.TR_SINGLE,size= (600, 300) ) 
		self.tree.Bind(wx.EVT_SET_FOCUS, self.onTreeSetFocus)
		self.tree.Bind(wx.EVT_CHAR, self.onTreeChar)
		self.treeRoot = self.tree.AddRoot("root")
		mainSizer.Add(self.tree,(1,0), (3,5), flag = wx.EXPAND|wx.ALL,border= 8) 
		mainSizer.Add(self.tree,(1,0), (3,5), flag = wx.EXPAND|wx.ALL,border= 8)
		sizer = wx.BoxSizer(wx.HORIZONTAL)
		# Translators: The label of an editable text field to filter the elements
		# in the browse mode Elements List dialog.
		label = wx.StaticText(self, wx.ID_ANY, NVDAString("&Filter by:"))
		sizer.Add(label)
		self.filterEdit = wx.TextCtrl(self, wx.ID_ANY)
		self.filterEdit.Bind(wx.EVT_TEXT, self.onFilterEditTextChange)
		sizer.Add(self.filterEdit)
		sizer.Add((40,-1)) 
		# Translators: The label of a button to activate an element
		# in the browse mode Elements List dialog.
		self.activateButton = wx.Button(self, wx.ID_ANY, NVDAString("&Activate"))
		self.activateButton.Bind(wx.EVT_BUTTON, lambda evt: self.onAction(True))
		sizer.Add(self.activateButton)
		# Translators: The label of a button to move to an element
		# in the browse mode Elements List dialog.
		self.moveButton = wx.Button(self, wx.ID_ANY, NVDAString("&Move to"))
		self.moveButton.Bind(wx.EVT_BUTTON, lambda evt: self.onAction(False))
		sizer.Add(self.moveButton)
		sizer.Add(wx.Button(self, wx.ID_CANCEL))
		mainSizer.Add(sizer,(4,0), (1,5), flag = wx.EXPAND|wx.ALL,border= 8)
		mainSizer.AddGrowableCol(1) 
		mainSizer.AddGrowableRow(2)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)

		self.tree.SetFocus()
		self.initElementType(self.ELEMENT_TYPES[self.lastSelectedElementType][0])
		self.Center(wx.BOTH | wx.CENTER_ON_SCREEN)

	def onElementTypeChange(self, evt):
		if self._timer:
			self._timer.Stop()
		elementType = self.childListBox.GetSelection()
		# We need to make sure this gets executed after the focus event.
		# Otherwise, NVDA doesn't seem to get the event.
		queueHandler.queueFunction(queueHandler.eventQueue, self.initElementType, self.ELEMENT_TYPES[elementType][0])
		self.lastSelectedElementType=elementType

	
	def initElementType(self, elType):
		if elType == "link":
			# Links can be activated.
			self.activateButton.Enable()
			self.SetAffirmativeId(self.activateButton.GetId())
		else:
			# No other element type can be activated.
			self.activateButton.Disable()
			self.SetAffirmativeId(self.moveButton.GetId())

		# Gather the elements of this type.
		self._elements = []
		self._initialElement = None

		parentElements = []
		isAfterSelection=False
		for item in self.document._iterNodesByType(elType):
			# Find the parent element, if any.
			for parent in reversed(parentElements):
				if item.isChild(parent.item):
					break
				else:
					# We're not a child of this parent, so this parent has no more children and can be removed from the stack.
					parentElements.pop()
			else:
				# No parent found, so we're at the root.
				# Note that parentElements will be empty at this point, as all parents are no longer relevant and have thus been removed from the stack.
				parent = None

			element=self.Element(item,parent)
			self._elements.append(element)

			if not isAfterSelection:
				isAfterSelection=item.isAfterSelection
				if not isAfterSelection:
					# The element immediately preceding or overlapping the caret should be the initially selected element.
					# Since we have not yet passed the selection, use this as the initial element. 
					try:
						self._initialElement = self._elements[-1]
					except IndexError:
						# No previous element.
						pass

			# This could be the parent of a subsequent element, so add it to the parents stack.
			parentElements.append(element)

		# Start with no filtering.
		self.filterEdit.ChangeValue("")
		self.filter("", newElementType=True)

		self.sayNumberOfElements()
		
	def sayNumberOfElements(self):
		def callback (count):
			self._timer = None
			if count:
				msg = _("%s elements") %str(count) if count > 1 else _("One element")
				queueHandler.queueFunction(queueHandler.eventQueue, speech.speakMessage, msg)
			else:
				queueHandler.queueFunction(queueHandler.eventQueue, speech.speakMessage, _("no element"))
		if self._timer:
			self._timer.Stop()
		
		self._timer = wx.CallLater(600,callback, self.tree.Count) 

	def filter(self, filterText, newElementType=False):
		# If this is a new element type, use the element nearest the cursor.
		# Otherwise, use the currently selected element.
		defaultElement = self._initialElement if newElementType else self.tree.GetItemPyData(self.tree.GetSelection())
		# Clear the tree.
		self.tree.DeleteChildren(self.treeRoot)

		# Populate the tree with elements matching the filter text.
		elementsToTreeItems = {}
		defaultItem = None
		matched = False
		#Do case-insensitive matching by lowering both filterText and each element's text.
		filterText=filterText.lower()
		for element in self._elements:
			if filterText and filterText not in element.item.label.lower():
				item = None
				continue
			matched = True
			parent = element.parent
			if parent:
				parent = elementsToTreeItems.get(parent)
			item = self.tree.AppendItem(parent or self.treeRoot, element.item.label)
			self.tree.SetItemPyData(item, element)
			elementsToTreeItems[element] = item
			if element == defaultElement:
				defaultItem = item

		self.tree.ExpandAll()

		if not matched:
			# No items, so disable the buttons.
			self.activateButton.Disable()
			self.moveButton.Disable()
			return

		# If there's no default item, use the first item in the tree.
		self.tree.SelectItem(defaultItem or self.tree.GetFirstChild(self.treeRoot)[0])
		# Enable the button(s).
		# If the activate button isn't the default button, it is disabled for this element type and shouldn't be enabled here.
		if self.AffirmativeId == self.activateButton.Id:
			self.activateButton.Enable()
		self.moveButton.Enable()

	def onTreeSetFocus(self, evt):
		# Start with no search.
		self._searchText = ""
		self._searchCallLater = None
		evt.Skip()

	def onTreeChar(self, evt):
		key = evt.KeyCode

		if key == wx.WXK_RETURN:
			# The enter key should be propagated to the dialog and thus activate the default button,
			# but this is broken (wx ticket #3725).
			# Therefore, we must catch the enter key here.
			# Activate the current default button.
			evt = wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, wx.ID_ANY)
			button = self.FindWindowById(self.AffirmativeId)
			if button.Enabled:
				button.ProcessEvent(evt)
			else:
				wx.Bell()

		elif key == wx.WXK_F2:
			item=self.tree.GetSelection()
			if item:
				selectedItemType=self.tree.GetItemPyData(item).item
				self.tree.EditLabel(item)
				evt.Skip()

		elif key >= wx.WXK_START or key == wx.WXK_BACK:
			# Non-printable character.
			self._searchText = ""
			evt.Skip()

		else:
			# Search the list.
			# We have to implement this ourselves, as tree views don't accept space as a search character.
			char = unichr(evt.UnicodeKey).lower()
			# IF the same character is typed twice, do the same search.
			if self._searchText != char:
				self._searchText += char
			if self._searchCallLater:
				self._searchCallLater.Restart()
			else:
				self._searchCallLater = wx.CallLater(1000, self._clearSearchText)
			self.search(self._searchText)

	def onTreeLabelEditBegin(self,evt):
		item=self.tree.GetSelection()
		selectedItemType = self.tree.GetItemPyData(item).item
		if not selectedItemType.isRenameAllowed:
			evt.Veto()

	def onTreeLabelEditEnd(self,evt):
			selectedItemNewName=evt.GetLabel()
			item=self.tree.GetSelection()
			selectedItemType = self.tree.GetItemPyData(item).item
			selectedItemType.rename(selectedItemNewName)

	def _clearSearchText(self):
		self._searchText = ""


	def search(self, searchText):
		item = self.tree.GetSelection()
		if not item:
			# No items.
			return

		# First try searching from the current item.
		# Failing that, search from the first item.
		items = itertools.chain(self._iterReachableTreeItemsFromItem(item), self._iterReachableTreeItemsFromItem(self.tree.GetFirstChild(self.treeRoot)[0]))
		if len(searchText) == 1:
			# If only a single character has been entered, skip (search after) the current item.
			next(items)

		for item in items:
			if self.tree.GetItemText(item).lower().startswith(searchText):
				self.tree.SelectItem(item)
				return

		# Not found.
		wx.Bell()

	def _iterReachableTreeItemsFromItem(self, item):
		while item:
			yield item

			childItem = self.tree.GetFirstChild(item)[0]
			if childItem and self.tree.IsExpanded(item):
				# Has children and is reachable, so recurse.
				for childItem in self._iterReachableTreeItemsFromItem(childItem):
					yield childItem

			item = self.tree.GetNextSibling(item)

	def onFilterEditTextChange(self, evt):
		self.filter(self.filterEdit.GetValue())
		evt.Skip()

	def onAction(self, activate):
		self.Close()
		# Save off the last selected element type on to the class so its used in initialization next time.
		self.__class__.lastSelectedElementType=self.lastSelectedElementType
		item = self.tree.GetSelection()
		item = self.tree.GetItemPyData(item).item
		if activate:
			item.activate()
		else:
			def move():
				speech.cancelSpeech()
				item.moveTo()
				item.report()
			wx.CallLater(100, move)

	


