#Manages configuration.
import os
import sys
from cStringIO import StringIO
from configobj import ConfigObj, ConfigObjError
from validate import Validator
from logHandler import log
import globalVars



# config section
SCT_General = "General"
# general section items
IT_ConfigVersion = "ConfigVersion"
IT_SkipEmptyParagraphs= "SkipEmptyParagraphs"
ID_UseQuickNavigationMode= "UseQuickNavigationMode"

GeneralConfSpec = """[{section}]
{configVersion} = string(default = "1.0")
	{skipEmptyParagraphs} = boolean(default=True)
	{useQuickNavigationMode} = boolean(default=False)
""".format(section = SCT_General,configVersion = IT_ConfigVersion,  skipEmptyParagraphs = IT_SkipEmptyParagraphs, useQuickNavigationMode = ID_UseQuickNavigationMode)

#: The configuration specification
#: @type: ConfigObj
confspec = ConfigObj(StringIO(
"""# appModule Configuration File
{0}
""".format(GeneralConfSpec)
), list_values=False, encoding="UTF-8")
confspec.newlines = "\r\n"


#: The active configuration, C{None} if it has not yet been loaded.
#: @type: ConfigObj
_conf = None
_configFileError = None
_val = Validator()


def load():
	global _conf, _configFileError
#	configFileName  = os.path.join(os.path.join(os.path.dirname(__file__), "..\\..").decode("mbcs") , "config\\settings")
	configFileName = os.path.join(globalVars.appArgs.configPath, "WordAddon.ini") 
	try:
		_conf = ConfigObj(configFileName, configspec = confspec, indent_type = "\t", encoding="UTF-8")
	except ConfigObjError as e:
		_conf = ConfigObj(None, configspec = confspec, indent_type = "\t", encoding="UTF-8")
		_conf.filename=configFileName
		_configFileError="Error parsing configuration file: %s" %e

	# Python converts \r\n to \n when reading files in Windows, so ConfigObj can't determine the true line ending.
	_conf.newlines = "\r\n"
	result = _conf.validate(_val)

	if not result or _configFileError:
		log.warn(configFileError)

def save():
	#Saves the configuration to the config file.
	#We never want to save config if runing securely
	if globalVars.appArgs.secure: return
	global _conf, _configFileError
	if _configFileError:
		raise RuntimeError("config file errors still exist")

	try:
		# Copy default settings and formatting.
		_conf.validate(_val, copy = True)
		_conf.write()
		
	except Exception, e:
		log.warning("Could not save configuration - probably read only file system")
		raise e


def toggleSkipEmptyParagraphsOption(toggle = True):
	global _conf
	if toggle:
		_conf[SCT_General][IT_SkipEmptyParagraphs] = not _conf[SCT_General][IT_SkipEmptyParagraphs]
		save()
	return _conf[SCT_General][IT_SkipEmptyParagraphs]

def toggleUseQuickNavigationModeOption(toggle = True):
	global _conf
	if toggle:
		_conf[SCT_General][ID_UseQuickNavigationMode] = not _conf[SCT_General][ID_UseQuickNavigationMode]
		save()
	return _conf[SCT_General][ID_UseQuickNavigationMode]
	