import api
import wx
import gui
import ui
from NVDAObjects.window.winword import WordDocument
from ww_wdConst import wdActiveEndPageNumber , wdFirstCharacterLineNumber , wdGoToHeading
from ww_collection import Collection, CollectionElement,ReportDialog
import time
from eventHandler import queueEvent
import addonHandler
addonHandler.initTranslation()
from NVDAObjects.window.winword import 		wdGoToNext,wdGoToHeading


def reportHeadingLevel(parent):
	app = parent.WinwordApplicationObject
	selection = app.selection
	doc = parent.doc = app.activeDocument
	theRange = doc.Range(selection.Start, selection.End)
	paragraphs =getattr(theRange,"paragraphs")
	if paragraphs.Count != 1:
		return
	paragraph = paragraphs[0]
	if paragraph.OutLineLevel >=10:
		return

	heading = Heading(parent, paragraph)
	heading.sayLevel()
		





		

	

class Heading(CollectionElement):
	def __init__(self, parent, item):
		super(Heading, self).__init__(parent, item)
		self.text = ""
		if item.range.text:
			self.text = item.range.text
			
		self.start = item.range.start
		self.level = item.OutLineLevel
		self.style = item.style.nameLocal
		r = self.parent.doc.range (self.start, self.start)
		self.line = r.information(wdFirstCharacterLineNumber )
		self.page = r.Information(wdActiveEndPageNumber )




	def sayLevel(self):
		ui.message(_("heading level %s") %self.level)

	
	def formatInfos(self):

		sInfo = _("""	Page {page}, line {line}
	Text: {text}
	Level: {level}
	Style: {style}
	""")
		sInfo = sInfo.replace("\n", "\r\n")
		return sInfo.format(page =self.page, line= self.line, text = self.text, level = self.level, style = self.style)
		

class Headings(Collection):
	_propertyName = ((None,Heading),)
	_name = (_("Heading"), _("Headings"))
	_wdGoToItem = wdGoToHeading

	def __init__(self, parent,obj, rangeType):
		self.rangeType = rangeType

		self.dialogClass = HeadingsDialog

		self.noElement = _("No heading")
		super(Headings, self).__init__( parent,obj)
	
	

	def getCollectionInRange(self, theRange):
		collection= []
		i = 0
		rangeObj = theRange
		while True:
			i = i+1
			wx.Yield()
			# stopped by user?
			if self.parent and self.parent.canceled:
				return []
			
			paragraph = rangeObj.Paragraphs[1]
			level = paragraph.OutLineLevel
			if level <10:
				collection.append((paragraph, Heading))
			newRangeObj=rangeObj.gotoNext(wdGoToHeading)
			if i > 400 or not newRangeObj or newRangeObj.start<=rangeObj.start:
				break
			rangeObj=newRangeObj



		return collection



				
class HeadingsDialog(ReportDialog):

	def __init__(self, parent,obj ):
		super(HeadingsDialog, self).__init__(parent,obj)
		
	def initializeGUI (self):
		self.lcLabel = _("Headings")
		self.lcColumns = (
			(_("Heading"), 100),
			(_("Text"),300),
			(_("Location"),150),
			(_("Level"),100),
			(_("Style"),200)
			)
		lcWidth = 0
		for column in self.lcColumns:
			lcWidth = lcWidth + column[1]
		
		self.lcSize = (lcWidth, self._defaultLCWidth)
		
		self.buttons = (
			(100, _("&Go to"),self.goTo),
			)


		self.tc1 = None
		self.tc2 = None



	def get_lcColumnsDatas(self, element):
		location = (_("Page {page}, line {line}")).format(page = element.page, line= element.line)
		index = self.collection.index(element)+1

		datas = ( index, element.text, location, element.level, element.style)

		return datas


		
	def onInputChar(self, evt):
		key = evt.GetKeyCode()
		if (key == wx.WXK_RETURN):
			self.goTo()
			return

	
		elif (key == wx.WXK_ESCAPE):
			self.Close()
			return

		evt.Skip()
