from logging import log
import config
import api
import wx
import gui
import ui
from NVDAObjects.window.winword import WordDocument
from ww_wdConst import wdActiveEndPageNumber , wdFirstCharacterLineNumber , wdGoToTable
from ww_collection import Collection, CollectionElement,ReportDialog
import time
from eventHandler import queueEvent
from logHandler import log
import addonHandler
addonHandler.initTranslation()


class Table(CollectionElement):
	def __init__(self, parent, item):

		super(Table, self).__init__(parent, item)
		self.columnsCount = item.Columns.Count
		self.rowsCount = item.Rows.Count
		#self.cells = item.Range.Cells
		r= item.Range

		#self.cellsObj = r.Cells
		#self.cells = []
		#for i in range(1,r.Cells.Count+1):
			#self.cells.append (Cell(self, r.Cells[i]))

		#self.descr = item.descr
		self.range = item.Range
		#self.title = item.title
		self.title = ""
		self.uniform = item.uniform
		self.start = item.range.Start
		r = self.parent.doc.range (self.start, self.start)
		self.line = r.information(wdFirstCharacterLineNumber )
		self.page = r.Information(wdActiveEndPageNumber )





	
	def formatInfos(self):
		sInfo = _("""Page {page}, line {line}
Title: {title}
Number of rows: {rowsCount}
Number of columns: {columnsCount}
""")
		sInfo = sInfo.replace("\n", "\r\n")
		return sInfo.format(page =self.page, line= self.line, title= self.title, rowsCount = self.rowsCount, columnsCount = self.columnsCount)

	def _getCellIndexInCollection(self, cell, collection):
		for i in range(0, len(collection)):
			element= collection[i]
			if (element.rowIndex == cell.rowIndex) and (element.columnIndex == cell.columnIndex):
				return i
		return None

	
	def moveToCell(self, type, whichOne , curCell ):
		(rowIndex, columnIndex) = (curCell.rowIndex, curCell.columnIndex)
		if  type == "row" :
			cells = self._getCellsOfRow(rowIndex)
			newCell = self._getCellInCollection(cells, whichOne, self._getCellIndexInCollection(curCell, cells))
			if newCell.columnIndex  == columnIndex:
				ui.message(_("Row's limit"))
				newCell = None

			
		elif type == "column" :
			cells = self._getCellsOfColumn(columnIndex)
			newCell = self._getCellInCollection(cells, whichOne, self._getCellIndexInCollection(curCell, cells))
			if newCell.rowIndex  == rowIndex:
				ui.message(_("Column's limit"))
				newCell = None
		
		else:
			# error
			return None
		
		if newCell == None:
			return None
			
		newCell.goTo()
		return newCell

	def _getCellInCollection(self, cells, whichOne, index):
		start = 0
		end = len(cells) -1

		if whichOne == "previous":
			if index == start:
				newIndex = start
			else:
				newIndex = index -1

		elif whichOne == "next":
			if index == end:
				newIndex = end
			else:
				newIndex = index + 1
		elif whichOne == "first":
			newIndex = start
		elif whichOne == "last":
			newIndex = end
		else:
			# error
			return None

		return cells[newIndex]
		
	def _getCellsOfRow(self, rowIndex):
		cells = []
		for i in range(1, self.columnsCount+1):

			try:
				cell = self.obj.cell(rowIndex, i)

			except:
				continue
			cells.append(Cell(self, cell))

		return cells
	


	def _getCellsOfColumn (self, columnIndex):
		cells = []
		for i in range(1, self.rowsCount+1):
			try:
				cell = self.obj.cell(i, columnIndex)

			except:
				continue
			cells.append(Cell(self, cell))



		return cells



	def sayElement(self, type, whichOne, currentCell, withFirst):
		if type == "row":
			self.sayRow(whichOne, currentCell, withFirst)
		elif type == "column":
			self.sayColumn(whichOne, currentCell, withFirst)
		elif type == "cell":
			self.sayCell(whichOne, currentCell, withFirst)
		else:
			# error
			log.error("SayElement invalid parameters %s" %type)
			return


	def sayCell(self,  whichOne = "current", currentCell= None, withFirst= False , columnHeader = None):
		if whichOne == "firstInRow":
			cells = self._getCellsOfRow(currentCell.rowIndex)
			newCell  = cells[0]
			
		elif  whichOne == "firstInColumn":
			cells = self._getCellsOfColumn(currentCell.columnIndex)
			newCell  = cells[0]

		elif whichOne == "lastInRow":
			cells = self._getCellsOfRow(currentCell.rowIndex)
			newCell  = cells[len(cells)-1]

		elif whichOne == "lastInColumn":
			cells = self._getCellsOfColumn(currentCell.columnIndex)
			newCell  = cells[len(cells)-1]			

			
			
		elif whichOne == "current":
			newCell = currentCell

		else:
			#error
			log.error ("error, sayCell invalid parameter %s" %whichOne)
			return
		if newCell:
			newCell.sayText(columnHeader )

		
	def sayRow(self, whichOne, currentCell, withFirst):
		if whichOne == "previous":
			index = currentCell.rowIndex-1

		elif whichOne == "next":
			index = currentCell.rowIndex +1
		elif whichOne == "current":
			index = currentCell.rowIndex
		else:
			#error
			log.error("sayRow invalid parameter")
			return

		cells = self._getCellsOfRow(index)
		ui.message(_("row %s") %index)
		for cell in cells:
			time.sleep(0.01)
			if withFirst and cell.rowIndex !=1:
				self.sayCell("firstInColumn", cell)
			
			self.sayCell("current", cell, False, True if not withFirst else None)

		
	def sayColumn (self, whichOne, currentCell, withFirst):
		if whichOne == "previous":
			index = currentCell.columnIndex-1

		elif whichOne == "next":
			index = currentCell.columnIndex +1
		elif whichOne == "current":
			index = currentCell.columnIndex
		else:
			#error
			log.error("sayColumn invalid parameter")
			return

		ui.message(_("column %s") %index)
		cells = self._getCellsOfColumn(index)

		for cell in cells:
			
			time.sleep(0.01)
			if withFirst and cell.columnIndex !=1:
				self.sayCell("firstInRow", cell)
				
			self.sayCell("current", cell, False, False if not withFirst else None)

class Tables(Collection):
	_propertyName = (("Tables",Table),)
	_name = (_("Table"), _("Tables"))
	_wdGoToItem = wdGoToTable
	def __init__(self, parent, obj, rangeType):
		self.rangeType = rangeType
		self.dialogClass = TablesDialog
		self.noElement = _("No table")
		super(Tables, self).__init__( parent, obj)
		
class TablesDialog(ReportDialog):

	def __init__(self, parent,obj ):
		super(TablesDialog, self).__init__(parent,obj)
		
	def initializeGUI (self):
		self.lcLabel = _("Tables")
		self.lcColumns = (
			(_("Table"), 100),
			(_("Location"),150),
			(_("Title"),300),
			(_("Number of rows"),100) ,
			(_("Number of columns"),100)
			)
		lcWidth = 0
		for column in self.lcColumns:
			lcWidth = lcWidth + column[1]
			
		self.lcSize = (lcWidth, self._defaultLCWidth)
		
		self.buttons = (
			(100, _("&Go to"),self.goTo),
			(101, _("&Delete"), self.delete),
			)


		self.tc1 = None
		self.tc2 = None



	def get_lcColumnsDatas(self, element):
		location = (_("Page {page}, line {line}")).format(page = element.page, line= element.line)
		index = self.collection.index(element)+1

		datas = (index,location, element.title, element.rowsCount, element.columnsCount)
		return datas



class Cell(CollectionElement):
	def __init__(self, parent,item ):
		
		super(Cell, self).__init__(parent, item)
		self.wordDocument = self.parent.parent
		self.columnIndex = item.ColumnIndex
		self.rowIndex = item.RowIndex
		"""
		self.height = item.Height
		self.width = item.Width
		self.heightRule = item.HeightRule #wdRowHeightRule
		self.nestingLevel = item.NestingLevel
		self.next = item.Next
		self.previous = item.Previous
		"""
		self.range = item.Range
		self.start = self.range.Start
		#self.tables = item.Tables
	
	def sayText (self, columnHeader = None):
		headerText = ""
		reportTableHeadersFlag = config.conf['documentFormatting']["reportTableHeaders"]

		if reportTableHeadersFlag and  columnHeader != None:
			headerText=self.wordDocument.fetchAssociatedHeaderCellText(self.obj,columnHeader)
			ui.message(headerText)


		text = self.obj.Range.Text
		text = text[:-2]
		if text != "":
			ui.message(text)
		else:
			ui.message(_("empty"))

	def moveToCell(self, type, whichOne ):
		row = Row(self, self.row)
		column = Column(self, self.column)
		if type == "row":
			if (whichOne in ["first", "previous"] and column.isFirst) or (whichOne in ["last", "next"] and column.isLast):
				#table's limit
				ui.message(_("table's limit"))
				return False
				
			newCell = row.getCell(whichOne, self.columnIndex)
			
		
		elif type == "column":
			if (whichOne in  ["first", "previous"] and row.isFirst) or (whichOne in ["last", "next"] and row.isLast):
				#table's limit
				ui.message(_("table's limit"))
				return False
				
			# move cell in column
			newCell = column.getCell(whichOne, self.rowIndex)
		else:
			# error
			log.error("moveToCell: invalid parameter %s" %type)
			
		if newCell:
			newCell.goTo()
			return newCell
	
		
		return False
